'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = recurseExtract;

var _of = require('./utils/of');

var _of2 = _interopRequireDefault(_of);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _RecursionBuilder = require('./RecursionBuilder');

var _RecursionBuilder2 = _interopRequireDefault(_RecursionBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Recurse a data structure call the extractor function to get the next recursive iteration object.
 *
 * @param {Function|String} extractor A function or string passed to `_.callback`.
 *   Used to extract the child of element
 * @param {Array<Object>|Object} items An item or list of items to iterate over
 * @returns {Iterartor} An iterator
 * @example
 *
 * let items = [{
 *   child: {}
 * }];
 *
 * for (let item of recurseExtract('child', items)) {
 *   console.log(item); //=> {}, {}
 * }
 */
function recurseExtract(extractor, items) {
  return _RecursionBuilder2.default.create((0, _of2.default)(items)).extractor(extractor).traverse(_isObject2.default);
}